@@ -124,6 +124,63 @@ ListState::ListState(const ListState &state) :
 
 }
 
+namespace
+{
+
+class GraphicTableCellStyle : public TableCellStyle
+{
+public:
+	GraphicTableCellStyle(const WPXPropertyList &xPropList, const char *psName);
+	virtual ~GraphicTableCellStyle();
+
+private:
+	virtual void writeCompat(OdfDocumentHandler *pHandler, const WPXPropertyList &propList) const;
+};
+
+GraphicTableCellStyle::GraphicTableCellStyle(const WPXPropertyList &xPropList, const char *const psName)
+	: TableCellStyle(xPropList, psName)
+{
+}
+
+GraphicTableCellStyle::~GraphicTableCellStyle()
+{
+}
+
+void GraphicTableCellStyle::writeCompat(OdfDocumentHandler *const pHandler, const WPXPropertyList &propList) const
+{
+	WPXPropertyList stylePropList;
+	WPXPropertyList::Iter i(propList);
+
+	/* first set padding, so that mPropList can redefine, if
+	   mPropList["fo:padding"] is defined */
+	stylePropList.insert("fo:padding", "0.0382in");
+	stylePropList.insert("draw:fill", "none");
+	stylePropList.insert("draw:textarea-horizontal-align", "center");
+
+	for (i.rewind(); i.next();)
+	{
+		if (strcmp(i.key(), "fo:background-color") == 0)
+		{
+			stylePropList.insert("draw:fill", "solid");
+			stylePropList.insert("draw:fill-color", i()->clone());
+		}
+		else if (strcmp(i.key(), "style:vertical-align")==0)
+			stylePropList.insert("draw:textarea-vertical-align", i()->clone());
+	}
+
+	pHandler->startElement("style:graphic-properties", stylePropList);
+	pHandler->endElement("style:graphic-properties");
+
+	// HACK to get visible borders
+	WPXPropertyList paraPropList;
+	paraPropList.insert("fo:border", "0.03pt solid #000000");
+
+	pHandler->startElement("style:paragraph-properties", paraPropList);
+	pHandler->endElement("style:paragraph-properties");
+}
+
+}
+
 class OdpGeneratorPrivate
 {
 public:
@@ -132,6 +189,7 @@ public:
 	/** update a graphic style element */
 	void _updateGraphicPropertiesElement(TagOpenElement &element, ::WPXPropertyList const &style, ::WPXPropertyListVector const &gradient);
 	void _writeGraphicsStyle();
+	void writeNotesStyles();
 	void _drawPolySomething(const ::WPXPropertyListVector &vertices, bool isClosed);
 	void _drawPath(const WPXPropertyListVector &path);
 
@@ -283,6 +341,69 @@ OdpGeneratorPrivate::~OdpGeneratorPrivate()
 	mFontManager.clean();
 }
 
+void OdpGeneratorPrivate::writeNotesStyles()
+{
+	{
+		WPXPropertyList styleProps;
+		styleProps.insert("style:name", "PresentationNotesPage");
+		styleProps.insert("style:family", "drawing-page");
+
+		mpHandler->startElement("style:style", styleProps);
+
+		WPXPropertyList pageProps;
+		pageProps.insert("presentation:display-header", "true");
+		pageProps.insert("presentation:display-footer", "true");
+		pageProps.insert("presentation:display-date-time", "true");
+		pageProps.insert("presentation:display-page-number", "false");
+
+		mpHandler->startElement("style:drawing-page-properties", pageProps);
+		mpHandler->endElement("style:drawing-page-properties");
+
+		mpHandler->endElement("style:style");
+	}
+
+	{
+		WPXPropertyList styleProps;
+		styleProps.insert("style:name", "PresentationNotesFrame");
+		styleProps.insert("style:family", "presentation");
+
+		mpHandler->startElement("style:style", styleProps);
+
+		WPXPropertyList graphicProps;
+		graphicProps.insert("draw:fill", "none");
+		graphicProps.insert("fo:min-height", "5in");
+
+		mpHandler->startElement("style:graphic-properties", graphicProps);
+		mpHandler->endElement("style:graphic-properties");
+
+		WPXPropertyList paraProps;
+		paraProps.insert("fo:margin-left", "0.24in");
+		paraProps.insert("fo:margin-right", "0in");
+		paraProps.insert("fo:text-indent", "0in");
+
+		mpHandler->startElement("style:para-properties", paraProps);
+		mpHandler->endElement("style:para-properties");
+
+		mpHandler->endElement("style:style");
+	}
+
+	{
+		WPXPropertyList styleProps;
+		styleProps.insert("style:name", "PresentationNotesTextBox");
+		styleProps.insert("style:family", "graphic");
+
+		mpHandler->startElement("style:style", styleProps);
+
+		WPXPropertyList graphicProps;
+		graphicProps.insert("draw:fill", "none");
+
+		mpHandler->startElement("style:graphic-properties", graphicProps);
+		mpHandler->endElement("style:graphic-properties");
+
+		mpHandler->endElement("style:style");
+	}
+}
+
 void OdpGeneratorPrivate::openListLevel(TagOpenElement *pListLevelOpenElement)
 {
 	if (!mListStates.top().mbListElementOpened.empty() &&
@@ -346,14 +467,18 @@ OdpGenerator::OdpGenerator(OdfDocumentHandler *pHandler, const OdfStreamType str
 	mpImpl->mpHandler->startDocument();
 	TagOpenElement tmpOfficeDocumentContent(mpImpl->getDocumentType().c_str());
 	tmpOfficeDocumentContent.addAttribute("xmlns:office", "urn:oasis:names:tc:opendocument:xmlns:office:1.0");
+	tmpOfficeDocumentContent.addAttribute("xmlns:presentation", "urn:oasis:names:tc:opendocument:xmlns:presentation:1.0");
 	tmpOfficeDocumentContent.addAttribute("xmlns:style", "urn:oasis:names:tc:opendocument:xmlns:style:1.0");
+	tmpOfficeDocumentContent.addAttribute("xmlns:table", "urn:oasis:names:tc:opendocument:xmlns:table:1.0");
 	tmpOfficeDocumentContent.addAttribute("xmlns:text", "urn:oasis:names:tc:opendocument:xmlns:text:1.0");
 	tmpOfficeDocumentContent.addAttribute("xmlns:draw", "urn:oasis:names:tc:opendocument:xmlns:drawing:1.0");
 	tmpOfficeDocumentContent.addAttribute("xmlns:dc", "http://purl.org/dc/elements/1.1/");
 	tmpOfficeDocumentContent.addAttribute("xmlns:svg", "urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0");
 	tmpOfficeDocumentContent.addAttribute("xmlns:fo", "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0");
 	tmpOfficeDocumentContent.addAttribute("xmlns:config", "urn:oasis:names:tc:opendocument:xmlns:config:1.0");
+	// WARNING: this is not ODF!
 	tmpOfficeDocumentContent.addAttribute("xmlns:ooo", "http://openoffice.org/2004/office");
+	tmpOfficeDocumentContent.addAttribute("xmlns:officeooo", "http://openoffice.org/2009/office");
 	tmpOfficeDocumentContent.addAttribute("office:version", "1.0");
 	if (mpImpl->mxStreamType == ODF_FLAT_XML)
 		tmpOfficeDocumentContent.addAttribute("office:mimetype", "application/vnd.oasis.opendocument.presentation");
@@ -454,6 +579,14 @@ OdpGenerator::~OdpGenerator()
 		}
 		mpImpl->mParagraphManager.write(mpImpl->mpHandler);
 		mpImpl->mSpanManager.write(mpImpl->mpHandler);
+
+		// writing out the table styles
+		for (std::vector<TableStyle *>::const_iterator iterTableStyles = mpImpl->mTableStyles.begin(); iterTableStyles != mpImpl->mTableStyles.end(); ++iterTableStyles)
+		{
+			(*iterTableStyles)->write(mpImpl->mpHandler);
+		}
+
+		mpImpl->writeNotesStyles();
 	}
 #ifdef MULTIPAGE_WORKAROUND
 	if ((mpImpl->mxStreamType == ODF_FLAT_XML) || (mpImpl->mxStreamType == ODF_STYLES_XML))
@@ -664,12 +797,10 @@ void OdpGenerator::setStyle(const ::WPXPropertyList &propList, const ::WPXProper
 
 void OdpGenerator::startLayer(const ::WPXPropertyList & /* propList */)
 {
-	mpImpl->mBodyElements.push_back(new TagOpenElement("draw:g"));
 }
 
 void OdpGenerator::endLayer()
 {
-	mpImpl->mBodyElements.push_back(new TagCloseElement("draw:g"));
 }
 
 void OdpGenerator::drawRectangle(const ::WPXPropertyList &propList)
@@ -1835,6 +1966,21 @@ void OdpGenerator::openTable(const ::WPXPropertyList &propList, const ::WPXPrope
 
 	mpImpl->mpCurrentTableStyle = pTableStyle;
 
+	// table must be inside a frame
+	TagOpenElement *pFrameOpenElement = new TagOpenElement("draw:frame");
+
+	pFrameOpenElement->addAttribute("draw:style-name", "standard");
+	if (propList["svg:x"])
+		pFrameOpenElement->addAttribute("svg:x", propList["svg:x"]->getStr());
+	if (propList["svg:y"])
+		pFrameOpenElement->addAttribute("svg:y", propList["svg:y"]->getStr());
+	if (propList["svg:width"])
+		pFrameOpenElement->addAttribute("svg:width", propList["svg:width"]->getStr());
+	if (propList["svg:height"])
+		pFrameOpenElement->addAttribute("svg:height", propList["svg:height"]->getStr());
+
+	mpImpl->mBodyElements.push_back(pFrameOpenElement);
+
 	TagOpenElement *pTableOpenElement = new TagOpenElement("table:table");
 
 	pTableOpenElement->addAttribute("table:name", sTableName.cstr());
@@ -1910,7 +2056,7 @@ void OdpGenerator::openTableCell(const ::WPXPropertyList &propList)
 
 	WPXString sTableCellStyleName;
 	sTableCellStyleName.sprintf( "%s.Cell%i", mpImpl->mpCurrentTableStyle->getName().cstr(), mpImpl->mpCurrentTableStyle->getNumTableCellStyles());
-	TableCellStyle *pTableCellStyle = new TableCellStyle(propList, sTableCellStyleName.cstr());
+	TableCellStyle *pTableCellStyle = new GraphicTableCellStyle(propList, sTableCellStyleName.cstr());
 	mpImpl->mpCurrentTableStyle->addTableCellStyle(pTableCellStyle);
 
 	TagOpenElement *pTableCellOpenElement = new TagOpenElement("table:table-cell");
@@ -1955,6 +2101,7 @@ void OdpGenerator::closeTable()
 	if (!mpImpl->mState.mInComment)
 	{
 		mpImpl->mBodyElements.push_back(new TagCloseElement("table:table"));
+		mpImpl->mBodyElements.push_back(new TagCloseElement("draw:frame"));
 	}
 }
 
@@ -1969,8 +2116,6 @@ void OdpGenerator::startComment(const ::WPXPropertyList &propList)
 	mpImpl->mState.mInComment = true;
 
 	TagOpenElement *const commentElement = new TagOpenElement("officeooo:annotation");
-	// WARNING: this is not ODF!
-	commentElement->addAttribute("xmlns:officeooo", "http://openoffice.org/2009/office");
 
 	// position & size
 	if (propList["svg:x"])
@@ -1994,7 +2139,7 @@ void OdpGenerator::endComment()
 	}
 
 	mpImpl->mState.mInComment = false;
-	mpImpl->mBodyElements.push_back(new TagCloseElement("office:annotation"));
+	mpImpl->mBodyElements.push_back(new TagCloseElement("officeooo:annotation"));
 }
 
 void OdpGenerator::startNotes(const ::WPXPropertyList &/*propList*/)
@@ -2005,11 +2150,35 @@ void OdpGenerator::startNotes(const ::WPXPropertyList &/*propList*/)
 		return;
 	}
 
-	mpImpl->mBodyElements.push_back(new TagOpenElement("presentation:notes"));
+	TagOpenElement *const notesElement = new TagOpenElement("presentation:notes");
+	notesElement->addAttribute("draw:style-name", "PresentationNotesPage");
+
+	mpImpl->mBodyElements.push_back(notesElement);
+
+	TagOpenElement *const thumbnailElement = new TagOpenElement("draw:page-thumbnail");
+	thumbnailElement->addAttribute("draw:layer", "layout");
+	thumbnailElement->addAttribute("presentation:class", "page");
+	// TODO: should the dimensions be hardcoded? If not, where
+	// should they come from?
+	thumbnailElement->addAttribute("svg:width", "5.5in");
+	thumbnailElement->addAttribute("svg:height", "4.12in");
+	thumbnailElement->addAttribute("svg:x", "1.5in");
+	thumbnailElement->addAttribute("svg:y", "0.84in");
+	WPXString pageNumber;
+	pageNumber.sprintf("%i", mpImpl->miPageIndex);
+	thumbnailElement->addAttribute("draw:page-number", pageNumber);
+
+	mpImpl->mBodyElements.push_back(thumbnailElement);
+	mpImpl->mBodyElements.push_back(new TagCloseElement("draw:page-thumbnail"));
 
 	TagOpenElement *const frameElement = new TagOpenElement("draw:frame");
+	frameElement->addAttribute("presentation:style-name", "PresentationNotesFrame");
 	frameElement->addAttribute("draw:layer", "layout");
 	frameElement->addAttribute("presentation:class", "notes");
+	frameElement->addAttribute("svg:width", "6.8in");
+	frameElement->addAttribute("svg:height", "4.95in");
+	frameElement->addAttribute("svg:x", "0.85in");
+	frameElement->addAttribute("svg:y", "5.22in");
 
 	mpImpl->mBodyElements.push_back(frameElement);
 
@@ -2026,6 +2195,8 @@ void OdpGenerator::endNotes()
 		return;
 	}
 
+	mpImpl->mState.mInNotes = false;
+
 	mpImpl->mBodyElements.push_back(new TagCloseElement("draw:text-box"));
 	mpImpl->mBodyElements.push_back(new TagCloseElement("draw:frame"));
 	mpImpl->mBodyElements.push_back(new TagCloseElement("presentation:notes"));
